@@ -3,6 +3,7 @@ from django.http import HttpResponse, HttpResponseRedirect, Http404, \
 from django.shortcuts import get_object_or_404, render
 from django.urls import reverse
 from django import forms
+from django.db import transaction
 from django.db.models import F, Min
 from django.forms.models import modelform_factory, modelformset_factory
 from django.views import generic
@@ -39,7 +40,7 @@ class ProgramFormMixin(generic.edit.FormMixin):
                                  slug=self.kwargs['form_slug'])
         self.program_form = form
         
-        if self.program_form.hidden() and 'sid' not in kwargs:
+        if self.program_form.hidden(): # and 'sid' not in kwargs:
             key, params = self.program_form.access_enable(), self.request.GET
             template = self.template_name
             if 'thanks' not in template and 'continue' not in template:
@@ -261,7 +262,7 @@ class SubmissionView(ProgramFormMixin, generic.UpdateView):
                         rec.number = F('number') - item._filesize
                         rec.save()
                         rec.refresh_from_db() # clear the decrementer
-                items.delete()
+                items.delete() # bulk is ok for ranked, because it's all of them
             else:
                 val = None
                 if block.block_type() == 'custom': val = block.default_value()
@@ -281,8 +282,6 @@ class SubmissionView(ProgramFormMixin, generic.UpdateView):
     
     def form_valid(self, form):
         if not self.page:
-            # TODO if program_form.status is not enabled
-            
             res = submission_handle_submit.send(self.program_form,
                                                 submission=self.object)
             if res and res[0][1]: return res[0][1]
@@ -336,9 +335,10 @@ class SubmissionView(ProgramFormMixin, generic.UpdateView):
             else: formset.save()
             
             items = self.object._items.filter(_block=formset.block.pk)
-            # these are the failed uploads:
-            items.filter(_file='', _filesize__gt=0).delete()
-        
+            failed_uploads = items.filter(_file='', _filesize__gt=0)
+            # must do one at a time, because of the ranked model:
+            for item in failed_uploads: item.delete()
+            
             types = {}
             for item in items:
                 if 'type' not in item._filemeta: continue
@@ -370,20 +370,31 @@ class SubmissionView(ProgramFormMixin, generic.UpdateView):
         return args
         
     def render_to_response(self, context):
+        form = self.program_form
+        
         if self.object._submitted:
-            if self.page or not self.program_form.review_after_submit():
+            if self.page or not form.review_after_submit():
                 url = reverse('form_thanks', kwargs=self.url_args(id=False))
                 return HttpResponseRedirect(url)
             else: context['submitted'] = True
         
-        if (self.page and context['page'] <= self.object._valid + 1
-         or self.object._valid == self.program_form.num_pages()
-         or self.object._submitted):
+        if self.page and form.status != Form.Status.ENABLED:
+            if not form.extra_time():
+                if self.object._valid == form.num_pages():
+                    url = reverse('submission_review', kwargs=self.url_args())
+                else: url = reverse('program',
+                                    kwargs={'slug': form.program.slug})
+                return HttpResponseRedirect(url)
+        
+        if (
+            self.page and context['page'] <= self.object._valid + 1
+            or self.object._valid == form.num_pages() or self.object._submitted
+        ):
             return super().render_to_response(context)
         
         # tried to skip ahead - go back to the last page that can be displayed
-        p, name, args = self.object._valid, 'submission_page', self.url_args()
-        if p: args['page'] = p + 1
+        name, args = 'submission_page', self.url_args()
+        if self.object._valid: args['page'] = self.object._valid + 1
         else: name = 'submission'
         
         return HttpResponseRedirect(reverse(name, kwargs=args))
@@ -417,7 +428,6 @@ class SubmissionBase(generic.View):
         self.program_form = form
         if not self.program_form.item_model: raise Http404()
         
-        # TODO: automated form close, timing here relative to rest?
         if form.status == Form.Status.DRAFT: return HttpResponseBadRequest()
         
         self.submission = get_object_or_404(self.program_form.model,
@@ -490,7 +500,13 @@ class SubmissionItemCreateView(SubmissionBase,
                                          _id=self.request.POST['item_id'])
             else:
                 nitems += 1
+            
             if self.block.max_items and nitems > self.block.max_items: break
+            if uploading and self.program_form.status == Form.Status.COMPLETED:
+                if 'item_id' in self.request.POST:
+                    ids.append(item._id)
+                    items.append(item)
+                break
             
             if not form.is_valid():
                 item._error = True
@@ -509,6 +525,7 @@ class SubmissionItemCreateView(SubmissionBase,
                     rec.number = F('number') - item._filesize
                     rec.save()
                     
+                    item._file, item._filesize = '', 0
                     delete_file(item._file)
                 
                 if name:
@@ -529,11 +546,19 @@ class SubmissionItemCreateView(SubmissionBase,
 
 
 class SubmissionItemBase(SubmissionBase):
-    def get_item(self):
+    def get_item(self, for_update=False):
         if 'item_id' not in self.request.POST: return HttpResponseBadRequest()
-        return get_object_or_404(self.program_form.item_model,
-                                 _submission=self.submission.pk,
-                                 _id=self.request.POST['item_id'])
+        
+        if for_update:
+            queryset = self.program_form.item_model.objects.select_for_update()
+        else: queryset = self.program_form.item_model.objects.all()
+        
+        try:
+            return queryset.get(_submission=self.submission.pk,
+                                _id=self.request.POST['item_id'])
+        except queryset.model.DoesNotExist:
+            name = queryset.model._meta.object_name
+            raise Http404(f'No {name} matches the given query.')
 
 
 class SubmissionItemUploadView(SubmissionItemBase):
@@ -567,31 +592,29 @@ class SubmissionItemUploadView(SubmissionItemBase):
         if not filetype_class and types: return HttpResponseBadRequest()
         
         msg_maxlen = SubmissionItem._message_maxlen()
-        def item_error(item, meta):
-            msg = meta['error']
-            item._error = True
-            item._message = msg[:msg_maxlen]
+        def item_error(item, msg):
+            item._error, item._message = True, msg[:msg_maxlen]
             delete_file(item._file)
+            item._file, item._filesize = '', 0
             return msg
         
         msg = ''
         if filetype_class:
             filetype = filetype_class()
             meta = filetype.meta(path)
-            if 'error' in meta: msg = item_error(item, meta)
+            if 'error' in meta: msg = item_error(item, meta['error'])
             else:
                 file_limits = block.file_limits()
                 if filetype.TYPE in file_limits:
                     msg = filetype.limit_error(meta, file_limits[filetype.TYPE])
-                    if msg:
-                        item._error = True
-                        item._message = msg[:msg_maxlen]
+                    if msg: item_error(item, msg)
                     else: msg = ''
                 
                 if not item._error:
                     opts = block.process_options(filetype.TYPE)
                     newmeta = filetype.process(item._file, meta, **opts)
-                    if 'error' in newmeta: msg = item_error(item, newmeta)
+                    if 'error' in newmeta:
+                        msg = item_error(item, newmeta['error'])
                     else:
                         if 'message' in newmeta:
                             warn_msg = newmeta.pop('message')
@@ -601,14 +624,14 @@ class SubmissionItemUploadView(SubmissionItemBase):
                         item._filemeta = newmeta
             item.save()
             
-            rec, created = SubmissionRecord.objects.get_or_create(
-                program=self.program_form.program, form=self.program_form.slug,
-                submission=self.submission.pk,
-                type=SubmissionRecord.RecordType.FILES
-            )
-            if created: rec.number = item._filesize
-            else: rec.number = F('number') + item._filesize
-            rec.save()
+        rec, created = SubmissionRecord.objects.get_or_create(
+            program=self.program_form.program, form=self.program_form.slug,
+            submission=self.submission.pk,
+            type=SubmissionRecord.RecordType.FILES
+        )
+        if created: rec.number = item._filesize
+        else: rec.number = F('number') + item._filesize
+        rec.save()
         
         return HttpResponse(msg)
 
@@ -617,18 +640,22 @@ class SubmissionItemRemoveView(SubmissionItemBase):
     http_method_names = ['post']
     
     def post(self, request, *args, **kwargs):
-        item = self.get_item()
-        if item._file:
+        file = None
+        with transaction.atomic():
+            item = self.get_item(for_update=True)
+            file, filesize = item._file, item._filesize
+            item.delete()
+        
+        if file:
             rec = SubmissionRecord.objects.get(
                 submission=self.submission.pk,
                 type=SubmissionRecord.RecordType.FILES
             )
-            rec.number = F('number') - item._filesize
+            rec.number = F('number') - filesize
             rec.save()
             
-            delete_file(item._file)
+            delete_file(file)
         
-        item.delete()
         return HttpResponse('')
 
 
